from SiemplifyAction import SiemplifyAction
from SiemplifyUtils import unix_now, convert_unixtime_to_datetime, output_handler
from ScriptResult import EXECUTION_STATE_COMPLETED, EXECUTION_STATE_FAILED, EXECUTION_STATE_TIMEDOUT
from CybereasonManager import CybereasonManager, CybereasonManagerNotFoundError
from TIPCommon import extract_configuration_param, extract_action_param
from constants import INTEGRATION_NAME, UPDATE_MALOP_STATUS_SCRIPT_NAME, STATUSES


@output_handler
def main():
    siemplify = SiemplifyAction()
    siemplify.script_name = UPDATE_MALOP_STATUS_SCRIPT_NAME
    siemplify.LOGGER.info("----------------- Main - Param Init -----------------")

    api_root = extract_configuration_param(siemplify, provider_name=INTEGRATION_NAME, param_name='API Root',
                                           is_mandatory=True)
    username = extract_configuration_param(siemplify, provider_name=INTEGRATION_NAME, param_name='Username',
                                           is_mandatory=True)
    password = extract_configuration_param(siemplify, provider_name=INTEGRATION_NAME, param_name='Password',
                                           is_mandatory=True)
    verify_ssl = extract_configuration_param(siemplify, provider_name=INTEGRATION_NAME, param_name='Verify SSL',
                                             default_value=False, input_type=bool)

    malop_guid = extract_action_param(siemplify, param_name='Malop ID', is_mandatory=True, print_value=True)
    malop_status = extract_action_param(siemplify, param_name="Status", is_mandatory=True, print_value=True)

    siemplify.LOGGER.info("----------------- Main - Started -----------------")

    status = EXECUTION_STATE_COMPLETED
    output_message = f"Successfully updated status for malop with ID {malop_guid} in {INTEGRATION_NAME}."
    result_value = True

    try:
        if malop_status not in STATUSES.keys():
            raise Exception(f"Status {malop_status} is invalid. Please enter one of the following: "
                            f"{', '.join(STATUSES.keys())}")

        manager = CybereasonManager(api_root=api_root, username=username, password=password, verify_ssl=verify_ssl,
                                    logger=siemplify.LOGGER, force_check_connectivity=True)
        manager.get_malop_or_raise(malop_guid)
        manager.update_malop_status(malop_guid, malop_status)

    except Exception as e:
        output_message = f"Error executing action {UPDATE_MALOP_STATUS_SCRIPT_NAME}. Reason: {e}"
        siemplify.LOGGER.error(output_message)
        siemplify.LOGGER.exception(e)
        status = EXECUTION_STATE_FAILED
        result_value = False

    siemplify.LOGGER.info("----------------- Main - Finished -----------------")
    siemplify.LOGGER.info(f"\n  status: {status}\n  is_success: {result_value}\n  output_message: {output_message}")
    siemplify.end(output_message, result_value, status)


if __name__ == "__main__":
    main()
